@@ -10,8 +10,7 @@ import wsgiref.simple_server
 
 from .request import VRequest
 from .response import VResponse
-from .loginmanager import VialLoginManager
-from .route import VRoute
+from .utils import format_status
 
 try:
     import nxtools
@@ -23,7 +22,8 @@ except ImportError:
 
 def format_traceback():
     exc_type, exc_value, tb = sys.exc_info()
-    result = "Traceback:\n\n    " +  "    ".join(traceback.format_exception(exc_type, exc_value, tb)[1:])
+    result = "Traceback:\n\n    " + \
+        "    ".join(traceback.format_exception(exc_type, exc_value, tb)[1:])
     return result
 
 
@@ -32,25 +32,35 @@ class VialRequestHandler(wsgiref.simple_server.WSGIRequestHandler):
         if not self.server.log_requests:
             return
         req, resp, _ = args
-        self.server.parent.logger.debug(f"[{resp}] {req} from {self.client_address[0]}")
+        self.server.parent.logger.debug(
+            f"[{resp}] {req} from {self.client_address[0]}"
+        )
 
 
 class Vial():
     def __init__(self, app_name="Vial", logger=None, **kwargs):
         # Server settings
         self.settings = {
-            "simple_response_always_200" : True,
-            "static_root" : None,
-            "static_index" : "index.html",
-            "static_404_to_index" : False
+            "simple_response_always_200": True,
+            "static_root": None,
+            "static_index": "index.html",
+            "static_404_to_index": False
         }
-        self.settings.update(kwargs)
 
         # Default headers
-        self.headers = {"Server" : app_name}
+        self.headers = {"Server": app_name}
         self.response = VResponse(self)
         self.app_name = app_name
         self._logger = logger
+        self.routes = []
+        self.setup()
+        self.settings.update(kwargs)
+
+    def __setitem__(self, key, value):
+        self.settings[key] = value
+
+    def __getitem__(self, key):
+        return self.settings[key]
 
     @property
     def logger(self):
@@ -65,30 +75,52 @@ class Vial():
     def __call__(self, environ, respond):
         request = VRequest(environ)
         try:
-            r = self.handle(request)
+
+            for rfunc, rpath, rkwargs in self.routes:
+                args = [r for r in rpath.split("/") if r]
+                route = request.route(*args)
+                if route:
+                    r = rfunc(request, **route.data)
+                    break
+            else:
+                r = self.handle(request)
 
             if type(r) == tuple and len(r) == 3:
                 status, headers, body = r
+
             elif r is None:
-                if self.settings["static_root"]:
+                if self["static_root"]:
                     status, headers, body = self.handle_static(request)
                 else:
                     status, headers, body = self.response(501)
+
             else:
-                self.logger.error(f"Vial.handle returned incorrect type {type(r)} for {request}")
+                self.logger.error(
+                    f"Vial.handle returned wrong type {type(r)} for {request}"
+                )
                 status, headers, body = self.response(500)
         except Exception:
             if has_nxtools:
                 nxtools.log_traceback()
             else:
-                self.logger.error(f"Unhandled exception")
+                self.logger.error("Unhandled exception")
                 self.logger.debug(format_traceback())
             status, headers, body = self.response(500)
 
         respond(
-            status,
+            format_status(status),
             [(key, str(value)) for key, value in headers.items() if value]
         )
+
+#        msg = \
+#            f"[{status}] {environ['REQUEST_METHOD']} {environ['PATH_INFO']}" \
+#            f" {environ['SERVER_PROTOCOL']} from {environ['REMOTE_ADDR']}"
+#
+#        if status < 400:
+#            self.logger.debug(msg)
+#        else:
+#            self.logger.error(msg)
+
         if request.method == "HEAD":
             yield b""
             return
@@ -98,41 +130,49 @@ class Vial():
         else:
             yield body
 
-
     def handle_static(self, request, root=None):
-        root = root or self.settings["static_root"]
+        root = root or self["static_root"]
         if isinstance(request, VRequest):
             rpath = request.path.lstrip("/")
         else:
             rpath = str(request).lstrip("/")
         if not rpath:
-            rpath = self.settings["static_index"]
+            rpath = self["static_index"]
         path = os.path.join(root, rpath)
 
         if not os.path.isfile(path):
-            index = os.path.join(root, self.settings["static_index"])
-            if self.settings["static_404_to_index"] and os.path.isfile(index):
+            index = os.path.join(root, self["static_index"])
+            if self["static_404_to_index"] and os.path.isfile(index):
                 path = index
             else:
                 return self.response(404, f"{rpath} cannot be found")
 
-        headers = { "Access-Control-Allow-Methods" : "GET" }
+        headers = {"Access-Control-Allow-Methods": "GET"}
         ct, enc = mimetypes.guess_type(path, strict=True)
         if ct:
             headers["Content-Type"] = ct
         if enc:
             headers["Content-Encoding"] = enc
 
-        #TODO: use generator
+        # TODO: use generator
         f = open(path, "rb")
         return self.response.raw(f.read(), headers=headers)
 
+    def route(self, function, path, **kwargs):
+        self.routes.append([function, path, kwargs])
 
-    def handle(self, request:VRequest):
-        return self.response(501)
+    def setup(self):
+        pass
 
+    def handle(self, request: VRequest):
+        pass
 
-    def serve(self, host:str="", port:int=8080, log_requests:bool=True):
+    def serve(
+            self,
+            host: str = "",
+            port: int = 8080,
+            log_requests: bool = True
+            ):
         """Start a development server"""
         self.logger.info(f"Starting HTTP server at {host}:{port}")
         server = wsgiref.simple_server.make_server(
@@ -154,4 +194,3 @@ class Vial():
             os.kill(os.getpid(), signal.CTRL_BREAK_EVENT)
         else:
             os.kill(os.getpid(), signal.SIGTERM)
-
